<?PHP
	defined( '_vrbxcv{_}+[-]/(=)25645ddgGGTUwk' ) or die( 'Restricted access' );

	$module_log	.= $boundary_delimiter . 'ipv4-6_address';
	
/**
 * Class to validate and to work with IPv6 addresses
 *
 * This was originally based on the PEAR class of the same name, but has been
 * entirely rewritten.
 *
 * https://github.com/jonphipps/requests/blob/master/Requests/IPv6.php eg 2a03:2880:3020:1ffd:face:b00c:0:8000
 *
 * @package Requests
 * @subpackage Utilities
 */
class IPv6Requests
{
/**
 * Uncompress an IPv6 address
 *
 * RFC 4291 allows you to compress concecutive zero pieces in an address to
 * '::'. This method expects a valid IPv6 address and expands the '::' to
 * the required number of zero pieces.
 *
 * Example:  FF01::101   ->  FF01:0:0:0:0:0:0:101
 *           ::1         ->  0:0:0:0:0:0:0:1
 *
 * @author Alexander Merz <alexander.merz@web.de>
 * @author elfrink at introweb dot nl
 * @author Josh Peck <jmp at joshpeck dot org>
 * @copyright 2003-2005 The PHP Group
 * @license http://www.opensource.org/licenses/bsd-license.php
 *
 * @param string $ip An IPv6 address
 * @return string The uncompressed IPv6 address
 */
 public static function uncompress($ip)
 {
   $c1 = -1;
   $c2 = -1;
   if (substr_count($ip, '::') === 1)
   {
     list($ip1, $ip2) = explode('::', $ip);
     if ($ip1 === ''){
       $c1 = -1;
     }
     else {
       $c1 = substr_count($ip1, ':');
     }
     if ($ip2 === ''){
       $c2 = -1;
     }
     else {
       $c2 = substr_count($ip2, ':');
     }
     if (strpos($ip2, '.') !== false){
       $c2++;
     }
     // ::
     if ($c1 === -1 && $c2 === -1){
       $ip = '0:0:0:0:0:0:0:0';
     }
     // ::xxx
     else if ($c1 === -1){
       $fill = str_repeat('0:', 7 - $c2);
       $ip = str_replace('::', $fill, $ip);
     }
     // xxx::
     else if ($c2 === -1){
       $fill = str_repeat(':0', 7 - $c1);
       $ip = str_replace('::', $fill, $ip);
     }
     // xxx::xxx
     else {
       $fill = ':' . str_repeat('0:', 6 - $c2 - $c1);
       $ip = str_replace('::', $fill, $ip);
     }
   }
   return $ip;
 }

 /**
  * Compress an IPv6 address
  *
  * RFC 4291 allows you to compress concecutive zero pieces in an address to
  * '::'. This method expects a valid IPv6 address and compresses consecutive
  * zero pieces to '::'.
  *
  * Example:  FF01:0:0:0:0:0:0:101   ->  FF01::101
  *           0:0:0:0:0:0:0:1        ->  ::1
  *
  * @see uncompress()
  * @param string $ip An IPv6 address
  * @return string The compressed IPv6 address
  */
 public static function compress($ip){
   // Prepare the IP to be compressed
   $ip = self::uncompress($ip);
   $ip_parts = self::split_v6_v4($ip);

   // Replace all leading zeros
   $ip_parts[0] = preg_replace('/(^|:)0+([0-9])/', '\1\2', $ip_parts[0]);
   // Find bunches of zeros
   if (preg_match_all('/(?:^|:)(?:0(?::|$))+/', $ip_parts[0], $matches, PREG_OFFSET_CAPTURE)){
     $max = 0;
     $pos = null;
     foreach ($matches[0] as $match){
       if (strlen($match[0]) > $max){
         $max = strlen($match[0]);
         $pos = $match[1];
       }
     }

     $ip_parts[0] = substr_replace($ip_parts[0], '::', $pos, $max);
   }

   if ($ip_parts[1] !== ''){
     return implode(':', $ip_parts);
   }
   else{
     return $ip_parts[0];
   }
 }

 /**
  * Splits an IPv6 address into the IPv6 and IPv4 representation parts
  *
  * RFC 4291 allows you to represent the last two parts of an IPv6 address
  * using the standard IPv4 representation
  *
  * Example:  0:0:0:0:0:0:13.1.68.3
  *           0:0:0:0:0:FFFF:129.144.52.38
  *
  * @param string $ip An IPv6 address
  * @return array [0] contains the IPv6 represented part, and [1] the IPv4 represented part
  */
 private static function split_v6_v4($ip){
   if (strpos($ip, '.') !== false){
     $pos = strrpos($ip, ':');
     $ipv6_part = substr($ip, 0, $pos);
     $ipv4_part = substr($ip, $pos + 1);
     return array($ipv6_part, $ipv4_part);
   }
   else {
     return array($ip, '');
   }
 }

 /**
  * Checks an IPv6 address
  *
  * Checks if the given IP is a valid IPv6 address
  *
  * @param string $ip An IPv6 address
  * @return bool true if $ip is a valid IPv6 address
  */
 public static function check_ipv6($ip){
   $ip = self::uncompress($ip);
   list($ipv6, $ipv4) = self::split_v6_v4($ip);
   $ipv6 = explode(':', $ipv6);
   $ipv4 = explode('.', $ipv4);
   if (count($ipv6) === 8 && count($ipv4) === 1 || count($ipv6) === 6 && count($ipv4) === 4){
     foreach ($ipv6 as $ipv6_part){
       // The section can't be empty
       if ($ipv6_part === '')       return false;

       // Nor can it be over four characters
       if (strlen($ipv6_part) > 4)  return false;

       // Remove leading zeros (this is safe because of the above)
       $ipv6_part = ltrim($ipv6_part, '0');
       if ($ipv6_part === '')       $ipv6_part = '0';

       // Check the value is valid
       $value = hexdec($ipv6_part);
       if (dechex($value) !== strtolower($ipv6_part) || $value < 0 || $value > 0xFFFF)  return false;
     }
     if (count($ipv4) === 4){
       foreach ($ipv4 as $ipv4_part){
         $value = (int) $ipv4_part;
         if ((string) $value !== $ipv4_part || $value < 0 || $value > 0xFF)   return false;
       }
     }
     return true;
   }
   else {
     return false;
   }
 }

/*
 * This address is an example IPv4 address:
 * IP v4 :
 * decimal :  80.229.244.167
 * binary  :  01010000111001011111010010100111
 * octal :  0120.0345.0364.0247
 * hexadecimal :  0x50.0xE5.0xF4.0xA7
 * long  :  1357247655
 *
 * IP v6 :
 * 6 to 4 address  :  2002:50E5:F4A7:0:0:0:0:0
 * :  2002:50E5:F4A7::
 * IPv4-mapped address :  0:0:0:0:0:FFFF:80.229.244.167
 * :  ::FFFF:80.229.244.167
 * :  ::FFFF:50E5:F4A7
 * IPv4-compatibility address  :  0:0:0:0:0:0:80.229.244.167
 * :  ::80.229.244.167
 * :  ::50E5:F4A7
 * long   :
 *
 * Net_IPv6::getAddressType() – Returns the type of an IP address
 * Synopsis
 * require_once 'Net/IPv6.php';
 * int Net_IPv6::getAddressType ( string $ip )

 * Description
 * RFC 1883, Section 2.3 describes several types of addresses in the IPv6 addresse space. This methods tries to find the type of address for the given IP.

 * Several address types are markers for reserved spaces and as consequence a subject to change.

 * Parameter
 * string $ip - the IP address in Hex format, compressed IPs are allowed

 * Return value
 * int - the addresstype

 * The type can be one of this constants:
 * NET_IPV6_MULTICAST
 * NET_IPV6_UNICAST_PROVIDER
 * NET_IPV6_LOCAL_LINK
 * NET_IPV6_LOCAL_SITE
 * NET_IPV6_UNKNOWN_TYPE
 * NET_IPV6_RESERVED_UNICAST_GEOGRAPHIC
 * NET_IPV6_RESERVED_IPX
 * NET_IPV6_RESERVED
 * NET_IPV6_RESERVED_NSAP
 * NET_IPV6_UNASSIGNED

 * An IPv6 address can have two formats:

 * Normal - Pure IPv6 format -     2001 : db8: 3333 : 4444 : 5555 : 6666 : 7777 : 8888
 * Dual - IPv6 plus IPv4 formats - 2001 : db8: 3333 : 4444 : 5555 : 6666 : 1 . 2 . 3 . 4
 *  http://stackoverflow.com/questions/15675347/ipv4-vs-ipv6-php-function
 */
 public static function isIPv6($ip) {
 /*
  * a version 6 IP address
  */
  if(strpos($ip, ":") !== false && strpos($ip, ".") === false) {
    if(filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6) === TRUE){
      return 'Pure'; //Pure format
    }
  }
  elseif(strpos($ip, ":") !== false && strpos($ip, ".") !== false){
    return 'Dual'; //dual format - last 4 values separated by '.' is ipv4 address
  }
  else{
   return 'Not v6';
  }
 }

 public static function isIPv4($ip) {
 /*
  * a version 4 IP address
  */
  $count = strpos($ip, ".");
  if($count > 1 ) { // must be more than one '.'
    if(filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4) === TRUE){
      return 'Pure'; //Pure format
    }
    else {
      return 'Range'; //Shortened
    }
  }
  else{
   return 'Not v4';
  }
 }
 public static function isIPpriv($ip) {
 /*
  * not a private IP address
  */
  $count = strpos($ip, ".");
  if($count > 1 ) { // must be more than one '.'
    if( filter_var ( $ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE ) === FALSE){
      return 'Private';
    }
    else {
      return 'Not Private';
    }
  }
 }
 public static function isIPres($ip) {
 /*
  * not a reserved IP address
  */
  $count = strpos($ip, ".");
  if($count > 1 ) { // must be more than one '.'
    if( filter_var ( $ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_RES_RANGE ) === FALSE){
      return 'Reserved';
    }
    else {
      return 'Not Reserved';
    }
  }
  else{
   return 'false';
  }
 }
}
/*
 * functions
 *
 * uncompress($ip)
 * compress($ip)
 * split_v6_v4($ip)
 * check_ipv6($ip)
 * isIPv6($ip)
 * isIPv4($ip)
 * isIPpriv($ip)
 * isIPres($ip)
 * 
 * 
 * 
 * 
 * 
 * From https://www.ip2location.com/developers/php
 *
 <?php
require_once 'IP2Location.php';

/*
   Cache whole database into system memory and share among other scripts & websites
   WARNING: Please make sure your system have sufficient RAM to enable this feature
*/
// $db = new \IP2Location\Database('./databases/IP-COUNTRY-SAMPLE.BIN', \IP2Location\Database::MEMORY_CACHE);

/*
   Cache the database into memory to accelerate lookup speed
   WARNING: Please make sure your system have sufficient RAM to enable this feature
*/
// $db = new \IP2Location\Database('./databases/IP-COUNTRY-SAMPLE.BIN', \IP2Location\Database::MEMORY_CACHE);


/*
	Default file I/O lookup
*/ /*
$db = new \IP2Location\Database('./databases/IP-COUNTRY-SAMPLE.BIN', \IP2Location\Database::FILE_IO);

$records = $db->lookup('8.8.8.8', \IP2Location\Database::ALL);

echo '<pre>';
echo 'IP Number             : ' . $records['ipNumber'] . "\n";
echo 'IP Version            : ' . $records['ipVersion'] . "\n";
echo 'IP Address            : ' . $records['ipAddress'] . "\n";
echo 'Country Code          : ' . $records['countryCode'] . "\n";
echo 'Country Name          : ' . $records['countryName'] . "\n";
echo 'Region Name           : ' . $records['regionName'] . "\n";
echo 'City Name             : ' . $records['cityName'] . "\n";
echo 'Latitude              : ' . $records['latitude'] . "\n";
echo 'Longitude             : ' . $records['longitude'] . "\n";
echo 'Area Code             : ' . $records['areaCode'] . "\n";
echo 'IDD Code              : ' . $records['iddCode'] . "\n";
echo 'Weather Station Code  : ' . $records['weatherStationCode'] . "\n";
echo 'Weather Station Name  : ' . $records['weatherStationName'] . "\n";
echo 'MCC                   : ' . $records['mcc'] . "\n";
echo 'MNC                   : ' . $records['mnc'] . "\n";
echo 'Mobile Carrier        : ' . $records['mobileCarrierName'] . "\n";
echo 'Usage Type            : ' . $records['usageType'] . "\n";
echo 'Elevation             : ' . $records['elevation'] . "\n";
echo 'Net Speed             : ' . $records['netSpeed'] . "\n";
echo 'Time Zone             : ' . $records['timeZone'] . "\n";
echo 'ZIP Code              : ' . $records['zipCode'] . "\n";
echo 'Domain Name           : ' . $records['domainName'] . "\n";
echo 'ISP Name              : ' . $records['isp'] . "\n";
echo '</pre>';
?>
 */